/*!
*
* RaphaëlSM : Raphaël extension for civil engineering mechanics drawings
*                      version 0.5.2 (2011-10-15)
*
*                 Copyright (C)  2011  Jan Stransky
*
*      Czech Technical University, Faculty of Civil Engineering,
*  Department of Structural Mechanics, 166 29 Prague, Czech Republic
*
*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @fileOverview <a href="http://raphaeljs.com/">Raphaël</a> (svg javascript library) extension by functions and "classes" for structural mechanics drawings (supports, deflections etc.). Requires <a href="http://raphaeljs.com/">raphael.js</a> and <a href="http://mech.fsv.cvut.cz/~stransky/software/raphaeltools/">raphaeltools.js</a> loaded before loading this file. For more information see <a href="http://mech.fsv.cvut.cz/~stransky/software/raphaelsm/">project homepage</a>.
<br /><br/>RaphaëlSM is a free software distributed under <a href='http://www.gnu.org/licenses/gpl.html'>GNU GPL license</a>.
* @author <a href="http://mech.fsv.cvut.cz/~stransky">Jan Stránský</a>
* @version Version 0.5.2 (2011-10-15)
*/




/** Abstract class for simple geometrical objects
* @class Abstract class for geometrical object
* @property {Raphael} rapheal raphael instance connected to the object
* @property {float} x x coordinate of the object
* @property {float} z z coordinate of the object
* @property {float} size size of the object
* @property {float} angle angle of the object
* @property {Array} pathList svg array (list) of the object
* @property {Raphael.el} obj actual raphael object
* @property {float} sin sin of this.angle
* @property {float} cos cos of this.angle*/
GeomObj = function() {

	/** Constructor
	* @param {Raphael} raphael raphael instance
	* @param {Object} parameters, see below
	* @param {float} params.x x coordinate [pixel]
	* @param {float} params.z z coordinate [pixel]
	* @param {float} params.size size [pixel]
	* @param {float} params.angle rotation angle from initial position*/
	this.constructorGeomObj = function(raphael,params) {
		this.raphael = raphael
		var params = params==undefined? {} : params;
		this.x = params.x==undefined? 0. : params.x;
		this.z = params.z==undefined? 0. : params.z;
		this.size = params.size==undefined? 1. : params.size;
		this.angle = params.angle==undefined? 0. : params.angle;
		this.pathList = null;
		this.obj = null;
		this.sin = 0.;
		this.cos = 0.;

		this.setSinCos();
		this.setPathList();
		this.makeObj(this.raphael);
		this.copies = [];
		this.copiesShifts = [];
	}

	/** Compute sin and cos from stored angle*/
	this.setSinCos = function() {
		this.sin = Math.sin(-this.angle*Math.PI/180);
		this.cos = Math.cos(-this.angle*Math.PI/180);
	}
	/** Set new this.pathList of receiver (each derived class must define its own setPathList method)*/
	this.setPathList = function() {}
	// Create a raphael object and assign it to this.obj
	this.makeObj = function() { this.obj = this.raphael.path(this.pathList); }
	// Set this.pathList to this.obj raphael object
	this.attrObjPath = function() { this.obj.attr({"path":this.pathList}); }
	/** Sets new x position of receiver (this.x = x)
	* @param {float} x new x coordinate [pixel]*/
	this.setX = function(x) { this.x = x; this.up(); }
	/** Sets new z position of receiver (this.z = z)
	* @param {float} z new z coordinate [pixel]*/
	this.setZ = function(z) { this.z = z; this.up(); }
	/** Sets new x and z position of receiver (this.x = x; this.z = z)
	* @param {float} x new x coordinate [pixel]
	* @param {float} z new z coordinate [pixel]*/
	this.setXZ = function(x,z) { this.x = x; this.z = z; this.up(); }
	/** Translate receiver in x direction (this.x += x)
	* @param {float} dx distance to be translated [pixel]*/
	this.translateX = function(dx) { this.x += dx; this.up(); }
	/** Translate receiver in x direction (this.z += dz)
	* @param {float} dz distance to be translated [pixel]*/
	this.translateZ = function(dz) { this.z += dz; this.up(); }
	/** Translate receiver in x and z direction (this.x += dx; this.z += dz)
	* @param {float} dx distance to be translated in x direction [pixel]
	* @param {float} dz distance to be translated in z direction [pixel]*/
	this.translate = function(dx,dz) { this.x += dx; this.z += dz; this.up(); }
	/** Rotate receiver by given angle (this.angle += phi)
	* @param {float} phi angle of rotation*/
	this.rotate = function(phi) { this.angle += phi; this.setSinCos(); this.up(); }
	/** Set angle of receiver (this.angle = phi)
	* @param {float} phi angle of rotation*/
	this.setAngle = function(phi) { this.angle = phi; this.setSinCos(); this.up(); }
	/** Copy receiver
	* @param {[TODO]} shifts TODO*/
	this.copy = function(shifts) { for (var i=0; i<shifts.length; i++) { this.copiesShifts.push(shifts[i]); this.copies.push(this.makeCopy()); this.up();} }
	/** Make a copy of itself (each derived class to be copied have to define its own makeCopy method)*/
	this.makeCopy = function() {}

	/** Update receiver (set this.pathList for given internal variables - x,z,angle... - as well as its copies)*/
	this.up = function() {
		this.setPathList(); this.attrObjPath();
		for (var c=0; c<this.copies.length; c++) { this.copies[c].setXZ(this.x+this.copiesShifts[c][0],this.z+this.copiesShifts[c][1]); }
	}
}

/** Arrow
* @class represents arrow
* @borrows GeomObj#setX as this.setX
* @borrows GeomObj#setZ as this.setZ
* @borrows GeomObj#setXZ as this.setXZ
* @borrows GeomObj#translateX as this.translateX
* @borrows GeomObj#translateZ as this.translateZ
* @borrows GeomObj#translate as this.translate
* @borrows GeomObj#rotate as this.rotate
* @borrows GeomObj#setAngle this.setAngle
* @borrows GeomObj#up as this.up
* @borrows GeomObj#rapheal as this.raphael
* @borrows GeomObj#x as this.x
* @borrows GeomObj#z as this.z
* @borrows GeomObj#size as this.size
* @borrows GeomObj#angle as this.angle
* @borrows GeomObj#pathList as this.pathList
* @borrows GeomObj#obj as this.obj
* @borrows GeomObj#sin as this.sin
* @borrows GeomObj#cos as this.cos
* @param {Raphael} raphael raphael instance
* @param {Object} params parameters, see below
* @param {float} [params.x=0.] x coordinate of arrow tip [pixel]
* @param {float} [params.z=0.] z coordinate of arrow tip [pixel]
* @param {float} [params.size=100.] length of the whole arrow (its size) [pixel]
* @param {float} [params.angle=0.] the arrow will be rotated by this angle [deg]
* @param {float} [params.headLen=size/4.] length of the arrow head [pixel]
* @param {float} [params.headWidth=size/16.] width of the arrow head [pixel]
* @property {float} headLen length of arrow head [pixel]
* @property {float} headWidth width of arrow head [pixel]*/
Arrow = function(raphael,params) {
	var params = params==undefined? {} : params;
	this.inheritFrom = GeomObj; this.inheritFrom();

	/** Constructor, see {@link Arrow} for parameters description*/
	this.constructorArrow = function(raphael,params) {
		this.constructorGeomObj(raphael,params);
		if (params.size==undefined) { this.size = 50.; }
		this.headLen = params.headLen==undefined? this.size/4. : params.headLen;
		this.headWidth = params.headWidth==undefined? this.size/16.: params.headWidth;
		this.up();
	}

	/** Sets svg path of receiver*/
	this.setPathList = function() {
		this.pathList = [
			["M",this.x,this.z],
			["L",this.x+this.cos*this.size,this.z+this.sin*this.size],
			["M",this.x+this.cos*this.headLen-this.sin*this.headWidth,this.z+this.sin*this.headLen+this.cos*this.headWidth],
			["L",this.x,this.z],
			["L",this.x+this.cos*this.headLen+this.sin*this.headWidth,this.z+this.sin*this.headLen-this.cos*this.headWidth]];
	}
	/** Resize receiver in both direction
	* @param {float} newSize new size of receiver*/
	this.resize = function(newSize) {
		if (Math.abs(newSize)>=1) { this.headLen *= newSize/this.size; this.headWidth *= newSize/this.size; this.size=newSize; this.up(); }
	}
	/** Returns copy of receiver
	* @returns {Arrow} copy of receiver*/
	this.makeCopy = function() {
		var ret = new Arrow(this.raphael,{x:this.x,z:this.z,size:this.size,angle:this.angle,headLen:this.headLen,headWidth:this.headWidth});
		ret.obj.remove();
		ret.obj = this.obj.clone();
		return ret;
	}

	this.constructorArrow(raphael,params);
};
/** Creates new Arrow object, for parameters meaning see {@link Arrow}
* @returns {Arrow} new Arrow object*/
Arrow.create = function(raphael,params) {
	return new Arrow(raphael,params);
}


/** Pin support 
* @class represents pin support
* @borrows GeomObj#setX as this.setX
* @borrows GeomObj#setZ as this.setZ
* @borrows GeomObj#setXZ as this.setXZ
* @borrows GeomObj#translateX as this.translateX
* @borrows GeomObj#translateZ as this.translateZ
* @borrows GeomObj#translate as this.translate
* @borrows GeomObj#rotate as this.rotate
* @borrows GeomObj#setAngle this.setAngle
* @borrows GeomObj#up as this.up
* @borrows GeomObj#rapheal as this.raphael
* @borrows GeomObj#x as this.x
* @borrows GeomObj#z as this.z
* @borrows GeomObj#size as this.size
* @borrows GeomObj#angle as this.angle
* @borrows GeomObj#pathList as this.pathList
* @borrows GeomObj#obj as this.obj
* @borrows GeomObj#sin as this.sin
* @borrows GeomObj#cos as this.cos
* @param {Raphael} raphael raphael instance
* @param {Object} params parameters, see below
* @param {float} [params.x=0.] x coordinate of supprt tip [pixel]
* @param {float} [params.z=0.] z coordinate of supprt tip [pixel]
* @param {float} [params.size=20.] height of the support [pixel]
* @param {float} [params.angle=0.] the support will be rotated by angle [deg]
* @param {bool|int} [params.sliding=true] if the support is sliding or not
* @param {float} [params.gap=5.] gap between triangle and line (sliding symbol) [pixel]
* @param {float} [params.ratio=0.7] width/height of the triangle [-]
* @property {bool} sliding true for sliding pins and false for fixed pins
* @property {float} ratio width/height of the pin
* @property {bool} sliding true for sliding pins and false for fixed pins*/
PinSupport = function(raphael,params) {
	var params = params==undefined? {} : params;
	this.inheritFrom = GeomObj; this.inheritFrom();

	/** Constructor, see {@link PinSupport} for parameters description*/
	this.constructorPinSupport = function(raphael,params) {
		this.constructorGeomObj(raphael,params);
		if (params.size==undefined) { this.size = 20.; }
		this.sliding = params.sliding==undefined? false : params.sliding
		this.ratio = params.ratio==undefined? .7 : params.ratio;
		this.gap = params.gap==undefined? 5. : params.gap;
		this.up();
	}

	/** Sets svg path of receiver*/
	this.setPathList = function() {
		if (this.sliding) {
			this.pathList = [
				["M",this.x,this.z],
				["L", this.x - this.cos*this.ratio*this.size - this.sin*this.size, this.z - this.sin*this.ratio*this.size + this.cos*this.size],
				["L", this.x + this.cos*this.ratio*this.size - this.sin*this.size, this.z + this.sin*this.ratio*this.size + this.cos*this.size],
				["Z"],
				["M", this.x - this.cos*this.ratio*this.size - this.sin*(this.size+this.gap),this.z - this.sin*this.ratio*this.size + this.cos*(this.size+this.gap)],
				["L", this.x + this.cos*this.ratio*this.size - this.sin*(this.size+this.gap),this.z + this.sin*this.ratio*this.size + this.cos*(this.size+this.gap)]];
		}
		else {
			this.pathList = [
				["M",this.x,this.z],
				["L", this.x - this.cos*this.ratio*this.size - this.sin*this.size, this.z - this.sin*this.ratio*this.size + this.cos*this.size],
				["L", this.x + this.cos*this.ratio*this.size - this.sin*this.size, this.z + this.sin*this.ratio*this.size + this.cos*this.size],
				["Z"]];
		}
	}
	/** Returns copy of receiver
	* @returns {PinSupport} copy of receiver*/
	this.makeCopy = function() {
		var ret = new PinSupport(this.raphael,{x:this.x,z:this.z,size:this.size,angle:this.angle,sliding:this.sliding,gap:this.gap,ratio:this.ratio});
		ret.obj.remove();
		ret.obj = this.obj.clone();
		return ret;
	}

	this.constructorPinSupport(raphael,params);
}

/** Creates new PinSupport object, for parameters meaning see {@link PinSupport}
* @returns {PinSupport} new PinSupport object*/
PinSupport.create = function(raphael,params) {
	return new PinSupport(raphael,params);
}



/** Fixed support 
* @class represents fixed support
* @borrows GeomObj#setX as this.setX
* @borrows GeomObj#setZ as this.setZ
* @borrows GeomObj#setXZ as this.setXZ
* @borrows GeomObj#translateX as this.translateX
* @borrows GeomObj#translateZ as this.translateZ
* @borrows GeomObj#translate as this.translate
* @borrows GeomObj#rotate as this.rotate
* @borrows GeomObj#setAngle this.setAngle
* @borrows GeomObj#up as this.up
* @borrows GeomObj#rapheal as this.raphael
* @borrows GeomObj#x as this.x
* @borrows GeomObj#z as this.z
* @borrows GeomObj#size as this.size
* @borrows GeomObj#angle as this.angle
* @borrows GeomObj#pathList as this.pathList
* @borrows GeomObj#obj as this.obj
* @borrows GeomObj#sin as this.sin
* @borrows GeomObj#cos as this.cos
* @param {Raphael} raphael  raphael instance
* @param {Object} params parameters, see below
* @param {float} [params.x=0.] x coordinate of the support center [pixel]
* @param {float} [params.z=0.] z coordinate of the support center [pixel]
* @param {float} [params.size=30.] size (height) of the support [pixel]
* @param {float} [params.angle=0.] the support will be rotated by angle [deg]
* @param {float} [params.ratio=2.] height/width (width of the 'flags')
* @param {int} [params.nFlags=5] number of 'flags'
* @param {bool|int} [params.reversed=false] reversed direction of 'flags'
* @property {float} ratio ratio between length of support and 'flag'
* @property {int} nFlags number of flags
* @property {bool} reversed if flags are reversed or not*/
FixedSupport = function(raphael,params) {
	var params = params==undefined? {} : params;
	this.inheritFrom = GeomObj; this.inheritFrom();

	/** Constructor, see {@link FixedSupport} for parameters description*/
	this.constructorFixedSupport = function(raphael,params) {
		this.constructorGeomObj(raphael,params);
		if (params.size==undefined) { this.size = 30.; }
		this.ratio = params.ratio==undefined? .2 : params.ratio;
		this.nFlags = params.nFlags==undefined? 5 : params.nFlags;
		this.reversed = params.reversed==undefined? false : params.reversed;
		this.up();
	}

	/** Sets svg path of receiver*/
	this.setPathList = function() {
		this.pathList = [
			["M", this.x - this.sin*this.size*.5, this.z + this.cos*this.size*.5],
			["L", this.x + this.sin*this.size*.5, this.z - this.cos*this.size*.5]];
		if (this.reversed) {
			for (var i=0; i<this.nFlags; i++) {
				var pt0 = vecTrans(0,this.size*(-.5+(i+1)/this.nFlags),this.sin,this.cos);
				var pt1 = vecTrans(-this.size*this.ratio,this.size*(-.5+i/this.nFlags),this.sin,this.cos);
				this.pathList.push(["M",this.x+pt0[0],this.z+pt0[1]]);
				this.pathList.push(["L",this.x+pt1[0],this.z+pt1[1]]);
			}
		}
		else {
			for (var i=0; i<this.nFlags; i++) {
				var pt0 = vecTrans(0,this.size*(-.5+i/this.nFlags),this.sin,this.cos);
				var pt1 = vecTrans(-this.size*this.ratio,this.size*(-.5+(i+1)/this.nFlags),this.sin,this.cos);
				this.pathList.push(["M",this.x+pt0[0],this.z+pt0[1]]);
				this.pathList.push(["L",this.x+pt1[0],this.z+pt1[1]]);
			}
		}
	}

	/** Returns copy of receiver
	* @returns {FixedSupport} copy of receiver*/
	this.makeCopy = function() {
		var ret = new FixedSupport(this.raphael,{x:this.x,z:this.z,size:this.size,angle:this.angle,ratio:this.ratio,nFlags:this.nFlags,reversed:this.reversed});
		ret.obj.remove();
		ret.obj = this.obj.clone();
		return ret;
	}

	this.constructorFixedSupport(raphael,params);
}
/** Creates new FixedSupport object, for parameters meaning see {@link PinSupport}
* @returns {FixedSupport} new FixedSupport object*/
FixedSupport.create = function(raphael,params) {
	return new FixedSupport(raphael,params);
}



/** Continuous load
* @class represents continuous load
* @param {Raphael} raphael raphael instance
* @param {Point2d} point1 1st point
* @param {Point2d} point2 2nd point
* @param {float} [valx=0.] value of load in x direction (global)
* @param {float} [valz=0.] value of load in z direction (global)
* @param {int} [nArrows=4] numer of arrows
* @property {Raphael} raphael raphael instance
* @property {Point2d} point1 1st point
* @property {Point2d} point2 2nd point
* @property {float} valx value in global x direction
* @property {float} valz value in global z direction
* @property {int} nArrows number of displayed arrows
* @property {Raphael.el} obj pointer to actual raphael object*/
ContinuousLoad = function(raphael,point1,point2,valx,valz,nArrows) {

	/** Constructor, see {@link ContinuousLoad} for parameters description*/
	this.constructorLine = function(raphael,point1,point2,valx,valz,nArrows) {
		this.raphael = raphael;
		this.point1 = point1;
		this.point2 = point2;
		this.valx = valx==undefined? 0. : valx;
		this.valz = valz==undefined? 0. : valz;
		this.nArrows = nArrows==undefined? 4 : nArrows;

		this.obj = this.raphael.path(this.pathList);
		this.up();
	}

	/** Sets new values to receiver
	* @param {float} [valx=0.] value in global x direction
	* @param {float} [valz=0.] value in global z direction*/
	this.setVals = function(valx,valz) {
		this.valx = valx==undefined? 0. : valx;
		this.valz = valz==undefined? 0. : valz;
		this.up();
	}

	/** Sets geometry proprties of receiver according to stored attributes*/
	this.setGeom = function() {
		this.x1 = this.point1.x;
		this.z1 = this.point1.z;
		this.x2 = this.point2.x;
		this.z2 = this.point2.z;
		this.dx = this.x2-this.x1;
		this.dz = this.z2-this.z1;
		this.len = Math.sqrt(this.dx*this.dx+this.dz*this.dz);
		this.cos = this.dx/this.len;
		this.sin = this.dz/this.len;
		this.lenVals = Math.sqrt(this.valx*this.valx+this.valz*this.valz);
		this.cosVal = this.valx/this.lenVals;
		this.sinVal = this.valz/this.lenVals;
	}

	/** Sets svg path of receiver*/
	this.setPathList = function() {
		this.pathList = [["M",this.x1,this.z1],["L",this.x2,this.z2],
				["L",this.x2-this.valx,this.z2-this.valz],["L",this.x1-this.valx,this.z1-this.valz],["Z"]];
		var segLen = this.len/(this.nArrows+1);
		for (var i=1; i<=this.nArrows; i++) {
			this.pathList.push(["M",this.x1-this.valx+i*segLen*this.cos,this.z1-this.valz+i*segLen*this.sin]);
			this.pathList.push(["L",this.x1+i*segLen*this.cos,this.z1+i*segLen*this.sin]);
			this.pathList.push(["M",
				this.x1+i*segLen*this.cos-this.cosVal*this.lenVals/4+this.sinVal*this.lenVals/16,
				this.z1+i*segLen*this.sin-this.sinVal*this.lenVals/4-this.cosVal*this.lenVals/16]);
			this.pathList.push(["L",this.x1+i*segLen*this.cos,this.z1+i*segLen*this.sin]);
			this.pathList.push(["L",
				this.x1+i*segLen*this.cos-this.cosVal*this.lenVals/4-this.sinVal*this.lenVals/16,
				this.z1+i*segLen*this.sin-this.sinVal*this.lenVals/4+this.cosVal*this.lenVals/16]);
		}
	}
	// Set this.pathList to this.obj raphael object
	this.attrObjPath = function() { this.obj.attr({"path":this.pathList}); }

	/** Update receiver (set this.pathList for given internal variables)*/
	this.up = function() { this.setGeom(); this.setPathList(); this.attrObjPath(); }

	this.constructorLine(raphael,point1,point2,valx,valz,nArrows);
}
/** Creates new ContinuousLoad object, for parameters meaning see {@link ContinuousLoad}
* @returns {ContinuousLoad} new ContinuousLoad object*/
ContinuousLoad.create = function(raphael,point1,point2,valx,valz,nArrows) {
	return new ContinuousLoad(raphael,point1,point2,valx,valz,nArrows);
}




/** Abstract class for curves (applied as internal forces, deflection etc.)
* @class represents generic curve object drawn over a line (e.g. beam)
* @param {Raphael} raphael raphael instance
* @param {Point2d} point1 first point of curve
* @param {Point2d} point2 second point of curve
* @param {Object} params parameters, see below
* @param {float} [params.scale=1.] scale scale between passed value and drawn value in pixels
* @param {bool} [params.closedBegin=true] if point1 is connected with appropriate value of curve
* @param {bool} [params.closedEnd=true] if point2 is connected with appropriate value of curve
* @param {bool} [params.closedBase=true] if point1 and point2 are connected within the curve definition
* @property {Raphael} raphael raphael instance
* @property {Point2d} point1 1st point
* @property {Point2d} point2 2nd point
* @property {float} scale scale scale between passed value and drawn value in pixels
* @property {bool} closedBegin if point1 is connected with appropriate value of curve
* @property {bool} closedEnd if point2 is connected with appropriate value of curve
* @property {bool} closedBase if point1 and point2 are connected within the curve definition
* @property {Raphael.el} actual raphael object
*/
CurveObj = function() {

	/** Constructor, see {@link LinearCurve} for parameters description*/
	this.constructorCurveObj = function(raphael,point1,point2,params) {
		var params = params==undefined? {} : params;
		this.raphael = raphael;
		this.point1 = point1;
		this.point2 = point2;
		this.scale = params.scale==undefined? 1. : params.scale;
		this.closedBegin = params.closedBegin==undefined? true : params.closedBegin;
		this.closedEnd = params.closedEnd==undefined? true : params.closedEnd;
		this.closedBase = params.closedBase==undefined? true : params.closedBase;

		this.setGeom()
		this.consts();
		this.setPathList();
		this.makeObj();
	}

	/** Sets function constants*/
	this.consts = function() {}

	/** Sets svg path list*/
	this.setPathList = function() {}

	/** Sets geometric parameters according to stored attributes*/
	this.setGeom = function() {
		this.x1 = this.point1.x;
		this.z1 = this.point1.z;
		this.x2 = this.point2.x;
		this.z2 = this.point2.z;
		this.dx = this.x2-this.x1;
		this.dz = this.z2-this.z1;
		this.len = Math.sqrt(this.dx*this.dx+this.dz*this.dz);
		this.segLen = this.len/(this.nSeg) || 1.
		this.cos = this.dx/this.len;
		this.sin = this.dz/this.len;
	}

	this.makeObj = function() { this.obj = this.raphael.path(this.pathList); }
	this.attrObjPath = function() { this.obj.attr({"path":this.pathList}); }

	/** Update*/
	this.up= function() { this.setGeom(); this.consts(); this.setPathList(); this.attrObjPath(); }

	this.closedPartsHandle = function(val1) {
		if (this.closedEnd) { this.pathList.push(["L",this.x2,this.z2]); }
		else { this.pathList.push(["M",this.x2,this.z2]); }
		if (this.closedBase) { this.pathList.push(["L",this.x1,this.z1]); }
		else { this.pathList.push(["M",this.x1,this.z1]); }
		if (this.closedBegin) { this.pathList.push(["L",this.x1-this.scale*this.sin*val1,this.z1+this.scale*this.cos*val1]); }
	}
}




/** Linear curve (e.g. shear force line)
* @class represents linear curve
* @param {Raphael} raphael raphael instance
* @param {Point2d} point1 1st point
* @param {Point2d} point2 2nd point
* @param {Object} params parameters, see below
* @param {float} [param.val1=0.] value in 1st point
* @param {float} [param.val2=0.] value in 2nd point
* @param {float} [param.scale=1.] scale factor
* @param {bool} [param.closedBegin=true] the line conecting 1st point with 1st nodal value
* @param {bool} [param.closedEnd=true] the line conecting 2nd point with 2nd nodal value
* @param {bool} [param.closedBase=true] the line conecting 1st and 2nd point
* @property {Raphael} raphael raphael instance
* @property {Point2d} point1 1st point
* @property {Point2d} point2 2nd point
* @property {float} scale scale scale between passed value and drawn value in pixels
* @property {bool} closedBegin if point1 is connected with appropriate value of curve
* @property {bool} closedEnd if point2 is connected with appropriate value of curve
* @property {bool} closedBase if point1 and point2 are connected within the curve definition
* @property {Raphael.el} obj pointer to actual raphael object
* @borrows CurveObj#setPathList as this.setPathList
* @borrows CurveObj#consts as this.consts
* @borrows CurveObj#up as this.up
* @property {float} val1 value in 1st point
* @property {float} val2 value in 2nd point*/
LinearCurve = function(raphael,point1,point2,params) {
	var params = params==undefined? {} : params;
	this.inheritFrom = CurveObj; this.inheritFrom();

	/** Constructor, see {@link LinearCurve} for parameters description*/
	this.constructorLinearCurve = function(raphael,point1,point2,params) {
		this.constructorCurveObj(raphael,point1,point2,params);
		this.val1 = params.val1==undefined? 0. : params.val1;
		this.val2 = params.val2==undefined? 0. : params.val2;
		this.up();
	}

	this.setPathList = function() {
		this.pathList =   [["M",this.x1-this.scale*this.sin*this.val1,this.z1+this.scale*this.cos*this.val1]];
		this.pathList.push(["L",this.x2-this.scale*this.sin*this.val2,this.z2+this.scale*this.cos*this.val2]);
		this.closedPartsHandle(this.val1);
	}

	/** Sets values of receiver
	* @param {float} val1 value in 1st point
	* @param {float} val2 value in 2nd point*/
	this.setVals = function(val1,val2) {
		this.val1 = val1;
		this.val2 = val2;
		this.up();
	}

	this.constructorLinearCurve(raphael,point1,point2,params);
}
/** Creates new LinearCurve object, for parameters meaning see {@link LinearCurve}
* @returns {LinearCurve} new LinearCurve object*/
LinearCurve.create = function(raphael,point1,point2,params) {
	return new LinearCurve(raphael,point1,point2,params);
}


/** Parabolic curve (e.g. bending moment line)
* @class represents parabolic curve
* @param {Raphael} raphael raphael instance
* @param {Point2d} point1 1st point
* @param {Point2d} point2 2nd point
* @param {Object} params parameters, see below
* @param {float} [params.val1=0.] value in 1st point
* @param {float} [params.valMid=0.] value in the middle of 1st and 2nd point
* @param {float} [params.val2=0.] value in 2nd point
* @param {float} [params.sclale=1.] scale factor
* @param {bool} [params.closedBegin=true] the line conecting 1st point with 1st nodal value
* @param {bool} [params.closedEnd=true] the line conecting 2nd point with 2nd nodal value
* @param {bool} [params.closedBase=true] the line conecting 1st and 2nd point
* @param {int} [params.nSeg=5] number of interpolating points
* @property {Raphael} raphael raphael instance
* @property {Point2d} point1 1st point
* @property {Point2d} point2 2nd point
* @property {float} scale scale scale between passed value and drawn value in pixels
* @property {bool} closedBegin if point1 is connected with appropriate value of curve
* @property {bool} closedEnd if point2 is connected with appropriate value of curve
* @property {bool} closedBase if point1 and point2 are connected within the curve definition
* @property {Raphael.el} obj pointer to actual raphael object
* @borrows CurveObj#setPathList as this.setPathList
* @borrows CurveObj#consts as this.consts
* @borrows CurveObj#up as this.up
* @property {float} val1 value in 1st point
* @property {float} val2 value in 2nd point
* @property {float} valMid value in the middle*/
ParabolicCurve = function(raphael,point1,point2,params) {
	var params = params==undefined? {} : params;
	this.inheritFrom = CurveObj; this.inheritFrom();
	this.constructorParabolicCurve = function(raphael,point1,point2,params) {
		this.constructorCurveObj(raphael,point1,point2,params);
		this.val1 = params.val1==undefined? 0. : params.val1;
		this.valMid = params.valMid==undefined? 0. : params.valMid;
		this.val2 = params.val2==undefined? 0. : params.val2;
		this.nSeg = params.nSeg==undefined? 5 : params.nSeg;
		this.up();
	}

	this.consts = function() {
		this.c0 = this.val1;
		this.c1 = (-3*this.val1 - this.val2 + 4*this.valMid)/this.len;
		this.c2 = (2*this.val1 + 2*this.val2 - 4*this.valMid)/this.len/this.len;
	}
	this.v = function(s) { return this.c0 + this.c1*s + this.c2*s*s; }
	this.dv = function(s) { return this.c1 + 2*this.c2*s; }
	this.setPathList = function() {
		this.pathList = [["M",this.x1-this.scale*this.sin*this.v(0),this.z1+this.scale*this.cos*this.v(0)]];
		var t = .3;
		for (var i=0; i<this.nSeg; i++) {
			var v1 = this.v(i*this.segLen);
			var v2 = this.v((i+1)*this.segLen);
			var dv1 = this.dv(i*this.segLen);
			var dv2 = this.dv((i+1)*this.segLen);
			var vec1 = vecTrans((i+t)*this.segLen,this.scale*(v1+dv1*t*this.segLen),this.sin,this.cos);
			var vec2 = vecTrans((i+1-t)*this.segLen,this.scale*(v2-dv2*t*this.segLen),this.sin,this.cos);
			var vec3 = vecTrans((i+1)*this.segLen,this.scale*v2,this.sin,this.cos);
			var ll = ["C",this.x1+vec1[0],this.z1+vec1[1],this.x1+vec2[0],this.z1+vec2[1],this.x1+vec3[0],this.z1+vec3[1]];
			this.pathList.push(ll);
		}
		this.closedPartsHandle(this.v(0));
	}

	/** Sets values of receiver
	* @param {float} val1 value in 1st point
	* @param {float} val2 value in 2nd point
	* @param {float} valMid value in the middle*/
	this.setVals = function(val1,val2,valMid) {
		this.val1 = val1;
		this.val2 = val2;
		this.valMid = valMid;
		this.up();
	}

	this.constructorParabolicCurve(raphael,point1,point2,params);
}
/** Creates new ParabolicCurve object, for parameters meaning see {@link ParabolicCurve}
* @returns {ParabolicCurve} new ParabolicCurve object*/
ParabolicCurve.create = function(raphael,point1,point2,params) {
	return new ParabolicCurve(raphael,point1,point2,params);
}



/** Generic class for 3rd and 4th order curves (used for deflection)
* @class represents generic class for deflection curve
* @param {Raphael} raphael raphael instance
* @param {Point2d} point1 1st point
* @param {Point2d} point2 2nd point
* @param {Object} params parameters, see below
* @param {float} [params.u1=0.] local x deflection in the 1st point
* @param {float} [params.w1=0.] local z deflection in the 1st point
* @param {float} [params.phi1=0.] rotation of the 1st point
* @param {float} [params.u2=0.] local x deflection in the 2nd point
* @param {float} [params.w2=0.] local z deflection in the 2nd point
* @param {float} [params.phi2=0.] rotation of the 2nd point
* @param {float} [params.wMid=0.] local z deflection from continuous load in the middle of the beam
* @param {float} [params.realLen=1.] real length (in units of u and w) of beam
* @param {float} [params.scale=1.] scale parameter for the deflection line
* @param {bool} [params.closedBegin=true] the line conecting 1st point with 1st nodal value
* @param {bool} [params.closedEnd=true] the line conecting 2nd point with 2nd nodal value
* @param {bool} [params.closedBase=true] the line conecting 1st and 2nd point
* @param {int} [params.nSeg=5] number of interpolation points
* @property {Raphael} raphael raphael instance
* @property {Point2d} point1 1st point
* @property {Point2d} point2 2nd point
* @property {float} scale scale scale between passed value and drawn value in pixels
* @property {bool} closedBegin if point1 is connected with appropriate value of curve
* @property {bool} closedEnd if point2 is connected with appropriate value of curve
* @property {bool} closedBase if point1 and point2 are connected within the curve definition
* @property {Raphael.el} obj pointer to actual raphael object
* @borrows CurveObj#setPathList as this.setPathList
* @borrows CurveObj#consts as this.consts
* @borrows CurveObj#up as this.up
* @property {float} u1 local x displacement of 1st point
* @property {float} w1 local z displacement of 1st point
* @property {float} phi1 rotation of 1st point
* @property {float} u2 local x displacement of 2nd point
* @property {float} w2 local z displacement of 2nd point
* @property {float} phi2 rotation of 2nd point
* @property {float} wMid local z deflection from continuous load in the middle of the beam
* @property {float} realLen real length (in units of u and w) of beam
* @property {float} scale parameter for the deflection line
* @property {int} nSeg number of interpolation points
*/
DeflCurve = function(raphael,point1,point2,params) {
	var params = params==undefined? {} : params;
	this.inheritFrom = CurveObj; this.inheritFrom();
	this.constructorDeflCurve = function(raphael,point1,point2,params) {
		this.constructorCurveObj(raphael,point1,point2,params);
		this.u1 = params.u1==undefined? 0. : params.u1;
		this.w1 = params.w1==undefined? 0. : params.w1;
		this.phi1 = params.phi1==undefined? 0. : params.phi1;
		this.u2 = params.u2==undefined? 0. : params.u2;
		this.w2 = params.w2==undefined? 0. : params.w2;
		this.phi2 = params.phi2==undefined? 0. : params.phi2;
		this.wMid = params.wMid==undefined? 0. : params.wMid;
		this.realLen = params.realLen==undefined? 1. : params.realLen;
		this.nSeg = params.nSeg==undefined? 5 : params.nSeg;
		this.scale = params.scale==undefined? 1. : params.scale;

		this.setGeom()
		this.lenScale = this.realLen/this.len;
		this.consts();
		this.setPathList();
		this.makeObj();
		this.up();
	}

	this.consts = function() {
		this.c0 = this.w1;
		this.c1 = -this.phi1*this.lenScale;
		this.c2 = (2*this.phi1+this.phi2)*this.lenScale/this.len + 3/this.len/this.len*(this.w2-this.w1) + 16/this.len/this.len*this.wMid;
		this.c3 = 2/this.len/this.len/this.len*(this.w1-this.w2) - 1/this.len/this.len*this.lenScale*(this.phi1+this.phi2) - 32/this.len/this.len/this.len*this.wMid;
		this.c4 = 16/this.len/this.len/this.len/this.len*this.wMid;
	}
	this.w = function(s) { return this.c0 + this.c1*s + this.c2*s*s + this.c3*s*s*s + this.c4*s*s*s*s; }
	this.dw = function(s) { return this.c1 + 2*this.c2*s + 3*this.c3*s*s + 4*this.c4*s*s*s; }
	this.setPathList = function() {
		this.pathList = [["M",this.x1+this.scale*(this.u1*this.cos-this.sin*this.w(0)),this.z1+this.scale*(this.u1*this.sin+this.cos*this.w(0))]];
		var t = .3;
		for (var i=0; i<this.nSeg; i++) {
			var w1 = this.w(i*this.segLen);
			var w2 = this.w((i+1)*this.segLen);
			var dw1 = this.dw(i*this.segLen);
			var dw2 = this.dw((i+1)*this.segLen);
			var vec1 = vecTrans(this.scale*this.u1+(i+t)*(this.segLen+this.scale*(this.u2-this.u1)/this.nSeg),this.scale*(w1+dw1*t*this.segLen),this.sin,this.cos);
			var vec2 = vecTrans(this.scale*this.u1+(i+1-t)*(this.segLen+this.scale*(this.u2-this.u1)/this.nSeg),this.scale*(w2-dw2*t*this.segLen),this.sin,this.cos);
			var vec3 = vecTrans(this.scale*this.u1+(i+1)*(this.segLen+this.scale*(this.u2-this.u1)/this.nSeg),this.scale*w2,this.sin,this.cos);
			var ll = ["C",this.x1+vec1[0],this.z1+vec1[1],this.x1+vec2[0],this.z1+vec2[1],this.x1+vec3[0],this.z1+vec3[1]];
			this.pathList.push(ll);
		}
		this.closedPartsHandle(this.w(0));
	}

	/** Sets new values of all stored displacements and rotations
	* @param {float} u1 local x deflection in the 1st point
	* @param {float} w1 local z deflection in the 1st point
	* @param {float} phi1 rotation of the 1st point
	* @param {float} u2 local x deflection in the 2nd point
	* @param {float} w2 local z deflection in the 2nd point
	* @param {float} phi2 rotation of the 2nd point
	* @param {float} wMid local z deflection from continuous load in the middle of the beam*/
	this.setDsplFull = function(u1,w1,phi1,u2,w2,phi2,wMid) {
		this.u1 = u1;
		this.w1 = w1;
		this.phi1 = phi1;
		this.u2 = u2;
		this.w2 = w2;
		this.phi2 = phi2;
		this.wMid = wMid || 0.;
		this.up();
	}
}

/* ***********************************************************
*
* specific higher order curves
*
************************************************************/

/** Cubic deflection (all 6 dofs can be set)
* @class represents cubic deflection, all 6 dofs can be set
* @param {Raphael} raphael raphael instance
* @param {Point2d} point1 1st point
* @param {Point2d} point2 2nd point
* @param {float} [u1=0.] local x deflection in the 1st point
* @param {float} [w1=0.] local z deflection in the 1st point
* @param {float} [phi1=0.] rotation of the 1st point
* @param {float} [u2=0.] local x deflection in the 2nd point
* @param {float} [w2=0.] local z deflection in the 2nd point
* @param {float} [phi2=0.] rotation of the 2nd point
* @param {float} [realLen=1.] real length (in units of u and w) of beam
* @param {float} [scale=1.] scale parameter for the deflection line
* @param {bool} [closedBegin=true] the line conecting 1st point with 1st nodal value
* @param {bool} [closedEnd=true] the line conecting 2nd point with 2nd nodal value
* @param {bool} [closedBase=true] the line conecting 1st and 2nd point
* @param {int} [nSeg=5] number of interpolation points
* @borrows CurveObj#setPathList as this.setPathList
* @borrows CurveObj#consts as this.consts
* @borrows CurveObj#up as this.up
* @property {float} u1 local x displacement of 1st point
* @property {float} w1 local z displacement of 1st point
* @property {float} phi1 rotation of 1st point
* @property {float} u2 local x displacement of 2nd point
* @property {float} w2 local z displacement of 2nd point
* @property {float} phi2 rotation of 2nd point
* @property {float} realLen real length (in units of u and w) of beam
* @property {int} nSeg number of interpolation points
*/
DeflCubic = function(raphael,point1,point2,params) {
	var params = params==undefined? {} : params;
	params.wMid = 0.;
	this.inheritFrom = DeflCurve; this.inheritFrom();
	this.constructorDeflCurve(raphael,point1,point2,params);
	/** Sets new values of all stored displacements and rotations
	* @param {float} u1 local x deflection in the 1st point
	* @param {float} w1 local z deflection in the 1st point
	* @param {float} phi1 rotation of the 1st point
	* @param {float} u2 local x deflection in the 2nd point
	* @param {float} w2 local z deflection in the 2nd point
	* @param {float} phi2 rotation of the 2nd point*/
	this.setDspl = function(u1,w1,phi1,u2,w2,phi2) { this.setDsplFull(u1,w1,phi1,u2,w2,phi2,0); }
}
/** Creates new DeflCubic object, for parameters meaning see {@link DeflCubic}
* @returns {DeflCubic} new DeflCubic object*/
DeflCubic.create = function(raphael,point1,point2,params) {
	return new DeflCubic(raphael,point1,point2,params);
}

/** 4th order deflection
* @class represents cubic deflection, all 6 dofs can be set
* @param {Raphael} raphael raphael instance
* @param {Point2d} point1 1st point
* @param {Point2d} point2 2nd point
* @param {Object} params parameters, see below
* @param {float} [params.u1=0.] local x deflection in the 1st point
* @param {float} [params.w1=0.] local z deflection in the 1st point
* @param {float} [params.phi1=0.] rotation of the 1st point
* @param {float} [params.u2=0.] local x deflection in the 2nd point
* @param {float} [params.w2=0.] local z deflection in the 2nd point
* @param {float} [params.phi2=0.] rotation of the 2nd point
* @param {float} [params.wMid=0.] local z deflection from continuous load in the middle of the beam
* @param {float} [params.realLen=1.] real length (in units of u and w) of beam
* @param {float} [params.scale=1.] scale parameter for the deflection line
* @param {bool} [params.closedBegin=true] the line conecting 1st point with 1st nodal value
* @param {bool} [params.closedEnd=true] the line conecting 2nd point with 2nd nodal value
* @param {bool} [params.closedBase=true] the line conecting 1st and 2nd point
* @param {int} [params.nSeg=5] number of interpolation points
* @borrows CurveObj#setPathList as this.setPathList
* @borrows CurveObj#consts as this.consts
* @borrows CurveObj#up as this.up
* @property {float} u1 local x displacement of 1st point
* @property {float} w1 local z displacement of 1st point
* @property {float} phi1 rotation of 1st point
* @property {float} u2 local x displacement of 2nd point
* @property {float} w2 local z displacement of 2nd point
* @property {float} phi2 rotation of 2nd point
* @property {float} realLen real length (in units of u and w) of beam
* @property {float} wMid local z deflection from continuous load in the middle of the beam
* @property {int} nSeg number of interpolation points
*/
Defl4thOrder = function(raphael,point1,point2,params) {
	var params = params==undefined? {} : params;
	this.inheritFrom = DeflCurve; this.inheritFrom();
	this.constructorDeflCurve(raphael,point1,point2,params);
	/** Sets new values of all stored displacements and rotations + wMid
	* @param {float} u1 local x deflection in the 1st point
	* @param {float} w1 local z deflection in the 1st point
	* @param {float} phi1 rotation of the 1st point
	* @param {float} u2 local x deflection in the 2nd point
	* @param {float} w2 local z deflection in the 2nd point
	* @param {float} phi2 rotation of the 2nd point
	* @param {float} wMid local z deflection from continuous load in the middle of the beam*/
	this.setDspl = function(u1,w1,phi1,u2,w2,phi2,wMid) { this.setDsplFull(u1,w1,phi1,u2,w2,phi2,wMid); }
}
/** Creates new Defl4thOrder object, for parameters meaning see {@link Defl4thOrder}
* @returns {Defl4thOrder} new Defl4thOrder object*/
Defl4thOrder.create = function(raphael,point1,point2,params) {
	return new Defl4thOrder(raphael,point1,point2,params);
}



/** Cubic deflection (no axial displacement)
* @class represents cubic deflection (no axial displacement)
* @param {Raphael} raphael raphael instance
* @param {Point2d} point1 1st point
* @param {Point2d} point2 2nd point
* @param {Object} params parameters, see below
* @param {float} [params.w1=0.] local z deflection in the 1st point
* @param {float} [params.phi1=0.] rotation of the 1st point
* @param {float} [params.w2=0.] local z deflection in the 2nd point
* @param {float} [params.phi2=0.] rotation of the 2nd point
* @param {float} [params.realLen=1.] real length (in units of u and w) of beam
* @param {float} [params.scale=1.] scale parameter for the deflection line
* @param {bool} [params.closedBegin=true] the line conecting 1st point with 1st nodal value
* @param {bool} [params.closedEnd=true] the line conecting 2nd point with 2nd nodal value
* @param {bool} [params.closedBase=true] the line conecting 1st and 2nd point
* @param {int} [params.nSeg=5] number of interpolation points
* @borrows CurveObj#setPathList as this.setPathList
* @borrows CurveObj#consts as this.consts
* @borrows CurveObj#up as this.up
* @property {float} w1 local z displacement of 1st point
* @property {float} phi1 rotation of 1st point
* @property {float} w2 local z displacement of 2nd point
* @property {float} phi2 rotation of 2nd point
* @property {float} realLen real length (in units of u and w) of beam
* @property {int} nSeg number of interpolation points
*/
DeflCubicNoU = function(raphael,point1,point2,params) {
	var params = params==undefined? {} : params;
	params.wMid = 0.;
	params.u1 = 0.;
	params.u2 = 0.;
	this.inheritFrom = DeflCurve; this.inheritFrom();
	this.constructorDeflCurve(raphael,point1,point2,params);
	/** Sets new values of w and phi in both points
	* @param {float} w1 local z deflection in the 1st point
	* @param {float} phi1 rotation of the 1st point
	* @param {float} w2 local z deflection in the 2nd point
	* @param {float} phi2 rotation of the 2nd point*/
	this.setDspl = function(w1,phi1,w2,phi2) { this.setDsplFull(0,w1,phi1,0,w2,phi2,0); }
}

/** Creates new DeflCubicNoU object, for parameters meaning see {@link DeflCubicNoU}
* @returns {DeflCubicNoU} new DeflCubicNoU object*/
DeflCubicNoU.create = function(raphael,point1,point2,params) {
	return new DeflCubicNoU(raphael,point1,point2,params);
}





/** 4th order deflection (no axial displacement)
* @class represents 4th order deflection (no axial displacement)
* @param {Raphael} raphael raphael instance
* @param {Point2d} point1 1st point
* @param {Point2d} point2 2nd point
* @param {Object} params parameters, see below
* @param {float} [params.w1=0.] local z deflection in the 1st point
* @param {float} [params.phi1=0.] rotation of the 1st point
* @param {float} [params.w2=0.] local z deflection in the 2nd point
* @param {float} [params.phi2=0.] rotation of the 2nd point
* @param {float} [params.wMid=0.] local z deflection from continuous load in the middle of the beam
* @param {float} [params.realLen=1.] real length (in units of u and w) of beam
* @param {float} [params.scale=1.] scale parameter for the deflection line
* @param {bool} [params.closedBegin=true] the line conecting 1st point with 1st nodal value
* @param {bool} [params.closedEnd=true] the line conecting 2nd point with 2nd nodal value
* @param {bool} [params.closedBase=true] the line conecting 1st and 2nd point
* @param {int} [params.nSeg=5] number of interpolation points
* @borrows CurveObj#setPathList as this.setPathList
* @borrows CurveObj#consts as this.consts
* @borrows CurveObj#up as this.up
* @property {float} w1 local z displacement of 1st point
* @property {float} phi1 rotation of 1st point
* @property {float} w2 local z displacement of 2nd point
* @property {float} phi2 rotation of 2nd point
* @property {float} realLen real length (in units of u and w) of beam
* @property {float} wMid local z deflection from continuous load in the middle of the beam
* @property {int} nSeg number of interpolation points
*/
Defl4thOrderNoU = function(raphael,point1,point2,params) {
	var params = params==undefined? {} : params;
	params.u1 = 0.;
	params.u2 = 0.;
	this.inheritFrom = DeflCurve; this.inheritFrom();
	this.constructorDeflCurve(raphael,point1,point2,params);
	/** Sets new values of w and phi in both points + wMid
	* @param {float} w1 local z deflection in the 1st point
	* @param {float} phi1 rotation of the 1st point
	* @param {float} w2 local z deflection in the 2nd point
	* @param {float} phi2 rotation of the 2nd point
	* @param {float} wMid local z deflection from continuous load in the middle of the beam*/
	this.setDspl = function(w1,phi1,w2,phi2,wMid) { this.setDsplFull(0,w1,phi1,0,w2,phi2,wMid); }
}
/** Creates new Defl4thOrderNoU object, for parameters meaning see {@link Defl4thOrderNoU}
* @returns {Defl4thOrderNoU} new Defl4thOrderNoU object*/
Defl4thOrderNoU.create = function(raphael,point1,point2,params) {
	return new Defl4thOrderNoU(raphael,point1,point2,params);
}
